from __future__ import annotations
from typing import TYPE_CHECKING, Optional, Union

from .partial_base import PartialBase

if TYPE_CHECKING:
    from corkus import Corkus
    from .identification_values import IdentificationValues
    from .identification_type import IdentificationType

class Identification(PartialBase):
    """Identification is a bonnus applied to an item to increse or decreese it's effectiveness."""
    def __init__(self, corkus: Corkus, type: IdentificationType, *, value: Optional[int] = None, values: Optional[IdentificationValues] = None):
        super().__init__(corkus)
        self._type = type
        self._value = value
        self._values = values

    @property
    def type(self) -> IdentificationType:
        """Type of the identification."""
        return self._type

    @property
    def value(self) -> Union[int, None]:
        """If this identification is returned from pre-identified item returns
        actually value of it. In other cases return ``None``."""
        return self._value

    @property
    def values(self) ->  Union[IdentificationValues, None]:
        """If this identification is in fact rolled and not static return
        it's possible values."""
        return self._values

    def __repr__(self) -> str:
        result = f"<Identification type={self.type.value!r}"
        if self.value is not None:
            result += f" value={self.value}"
        if self.values is not None:
            result += f" values=\"{self.values.min}/{self.values.max}\""
        return result + ">"



from aiohttp import ClientResponse

class CorkusException(Exception):
    """Base exception class for Corkus.py

    Ideally speaking, this could be caught to handle any exceptions thrown from this library.
    """

class InvalidInputError(CorkusException):
    """Exception that's thrown when you call a corkus function with
    invalid arguments. Corkus verified that argument isn't valid
    before sending request."""

class CorkusTimeoutError(CorkusException):
    """Exception that's thrown when a request timeouts."""
    def __init__(self, timeout: int, url: str) -> None:
        super().__init__(f"request timeout after {timeout} seconds for {url}")
        self._timeout = timeout
        self._url = url

    @property
    def timeout(self) -> int:
        """Configured timeout."""
        return self._timeout

    @property
    def url(self) -> int:
        """URL for which timeout happened."""
        return self._timeout

class HTTPError(CorkusException):
    """Exception that's thrown when an HTTP request operation fails."""
    def __init__(self, response: ClientResponse) -> None:
        super().__init__(f"received status code: HTTP {response.status} for {response.url}")
        self._response = response

    @property
    def response(self) -> ClientResponse:
        """Failed response returned by AIOHTTP.

        .. |ClientResponse| replace:: ``ClientResponse``
        .. _ClientResponse: https://docs.aiohttp.org/en/latest/client_reference.html#aiohttp.ClientResponse

        :rtype: |ClientResponse|_
        """
        return self._response

class WynncraftServerError(HTTPError):
    """Exception that's thrown for when a 500 range status code occurs.

    This error indicates an unexpected error prevent the server from processing the request.
    If the error continues occurring for the same request it should be reported on the
    `Wynncraft API issue tracker <https://github.com/Wynncraft/WynncraftAPI/issues>`_;
    or if you believe the error is exploitable, or directly to Colin (Colin#0670 on discord, colin350 on the forums)
    if the issue is exploitable/poses a risk to the stability of the API.
    """

class RatelimitExceeded(HTTPError):
    """Exception that's thrown for when a 429 status code occurs.

    .. warning::

        This exception shouldn't be thrown when using Corkus normally.
        If you see that error it's most likely because you disabled :ref:`ratelimit`.
        If you are sure that rate limit is enabled please
        `create a bug repport <https://github.com/MrBartusek/corkus.py/issues/new/choose>`_

    .. danger::

        IPs that repeatedly exceed the rate limit could be blacklisted.
    """

class BadRequest(HTTPError):
    """Exception that's thrown for when a 400 status code occurs.

    Indicates that a parameter given by the client was in an incorrect format,
    or that the requested resource doesn't exist. This client should not
    immediately request this resource again without changes.
    """



from __future__ import annotations
from .base import CorkusBase
from .skill_points import SkillPoints

class IngredientItemModifiers(CorkusBase):
    """Modifier of :py:class:`Ingredient` applied to regular crafted items."""
    @property
    def durability(self) -> int:
        """Modifier on item durability."""
        return self._attributes.get("durabilityModifier", 0)

    @property
    def skill_points(self) -> SkillPoints:
        """Number of required skill points added to the item."""
        return SkillPoints(
            corkus = self._corkus,
            strength = self._attributes.get("strengthRequirement", 0),
            dexterity = self._attributes.get("dexterityRequirement", 0),
            intelligence = self._attributes.get("intelligenceRequirement", 0),
            defence = self._attributes.get("defenceRequirement", 0),
            agility = self._attributes.get("agilityRequirement", 0)
        )

    def __repr__(self) -> str:
        return f"<IngredientComsumableModifiers durability={self.durability} skill_points={self.skill_points}>"



from __future__ import annotations
from typing import Union
from .base import CorkusBase
from .partial_server import PartialServer

class PlayerStatus(CorkusBase):
    """Represents online status of :py:class:`Player`."""
    @property
    def online(self) -> bool:
        """Is player online right now."""
        return self._attributes.get("online", False)

    @property
    def server(self) -> Union[PartialServer, None]:
        """The server that this player is currently on, ``None`` if player is offline."""
        if self._attributes.get("server") is not None:
            return PartialServer(self._corkus, self._attributes.get("server", ""))
        else:
            return None

    def __repr__(self) -> str:
        return f"<PlayerStatus online={self.online}>"



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from corkus import Corkus
    from corkus.utils.request import CorkusRequest

class Endpoint():
    def __init__(self, corkus: Corkus, request: CorkusRequest) -> None:
        self._corkus = corkus
        self._request = request
